#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pygame
from pygame.locals import *

import gc
gc.enable()
VELOCIDAD = 40

import asyncore
import asynchat
import socket
import operator
import os
import sys

from Tanque import Tanque
from Bala import Bala

RESOLUCION_MONITOR = (800,600)
TERMINATOR = "\n"
PUERTO = 5000

class JAMtank(asynchat.async_chat):

	def __init__(self, ip):

		self.ip_puerto = (ip, PUERTO)

		# Siempre soy un Cliente en la red
		asynchat.async_chat.__init__(self)

		self.tanques = pygame.sprite.RenderUpdates()
		self.protagonista = None

		self.jugadores = {} # Diccionario=> nombre:(angulo,x,y)

		self.set_terminator(TERMINATOR)
		self.create_socket(socket.AF_INET, socket.SOCK_DGRAM)
		self.connect((ip, PUERTO))

		self.buffer_de_entrada = ""
		self.nombre = os.getpid()
		self.buffer_de_salida = "hello %d%s" % (self.nombre, TERMINATOR)

		self.handle_write() # Si hay datos en buffer_de_salida, escribirlos en el socket

		self.mis_balas = pygame.sprite.RenderUpdates()
		self.todas_las_balas = pygame.sprite.RenderUpdates()

		self.resolucion_monitor = RESOLUCION_MONITOR # tupla (ancho, alto)
		self.ventana = None
		self.clock = None
		self.fondo = None
		self.estado = None
		self.sonido_disparo = None

		self.Run() # ejecuta el juego. Para mantener compatibilidad con adaptador de sugar hay que eliminar este llamado.

	# --------------------- La Red -------------------------------------------------
    	def writable(self):
        	return bool(self.buffer_de_salida) # ¿Hay datos en el buffer de salida?, si hay llamar a handle_write
    	def handle_write(self):
		sent = self.send(self.buffer_de_salida) # escribe en el socket
		#print "Buffer de salida en Cliente:", self.buffer_de_salida
		self.buffer_de_salida = ""#self.buffer_de_salida[sent:] # limpia el buffer
		
	def collect_incoming_data(self, datos):
		'''Llamado cuando se reciben datos, los agrega al buffer_de_entrada. '''
		self.buffer_de_entrada = datos
	def found_terminator(self):
		''' Se llama cuando en el buffer_de_entrada de datos se encuentra el caracter establecido en set_terminator(). '''

		valores = self.buffer_de_entrada.split()

		#print "Entrada de datos al Cliente", valores

		nombre = int(valores[0])
		tipo = str(valores[1])
		angulo = int(valores[2])
		x = int(valores[3])
		y = int(valores[4])

		if tipo == "Tanque":
			if nombre not in self.jugadores:
			# Guarda el tanque en el diccionario
				if nombre == self.nombre: # Si es mi tanque
					imagen = "Tanque.bmp"
					self.protagonista = self.agrega_tanque(imagen, nombre, angulo, x, y)
				else: # Si no es mi tanque
					imagen = "Tanque_Enemigo.bmp"
					self.agrega_tanque(imagen, nombre, angulo, x, y)

				print "Nuevo Tanque en la Red: ", self.jugadores.items()

			elif nombre in self.jugadores and nombre != self.nombre:
			# Actualiza los tanques ajenos en el diccionario
				self.jugadores[nombre].set_posicion(angulo, x, y) # actualiza la posición

		'''
		if tipo == "bye":
			self.jugadores[nombre].remove(self.tanques)
			del self.jugadores[nombre] '''	

		if tipo == "Bala" and nombre != self.nombre:
			self.crear_disparo(nombre, angulo, (x, y))
			#print "Llegando una bala al cliente"

		#if tipo == "Bala" and nombre == self.nombre: print "Llegando mi bala al cliente", valores

		self.buffer_de_entrada = ""

	def agrega_tanque(self, imagen, nombre, angulo, x, y):
	# Agrega los tanques a los grupos de sprites y al diccionario de tanques en la red
		tanque_nuevo = Tanque(imagen, self.resolucion_monitor)
		tanque_nuevo.set_posicion(angulo, x, y)
		self.jugadores[nombre] = tanque_nuevo
		self.tanques.add(tanque_nuevo)
		return tanque_nuevo

	def crear_disparo(self, autor, angulo, origen_disparo):
		# Genera un disparo
		#print "Disparo en el Juego: ", autor, angulo, origen_disparo
		self.sonido_disparo.play()
		if int(autor) == self.nombre:
			bala = Bala(autor, angulo, origen_disparo)
			#self.buffer_de_salida += "%s %s %s %s %s%s" % (self.nombre, "Bala", angulo, origen_disparo[0], origen_disparo[1], TERMINATOR)
			self.mis_balas.add(bala) # solo para mantener referencia a las balas propias
			self.todas_las_balas.add(bala)
		else:
			bala = Bala(autor, angulo, origen_disparo)
			self.todas_las_balas.add(bala)
			


	# ----------------------- Comienza el juego pygame ---------------------------
	def Run(self):

		self.set_entorno()
		self.ventana.blit(self.fondo, (0,0))
		pygame.display.update()

		print "*** Entrando en Bucle Principal . . ."
		self.estado = "En Juego"

		while self.estado == "En Juego":
			asyncore.loop(timeout=0.2, count=1) # bucle de la red

			self.clock.tick(VELOCIDAD)

			if self.protagonista:
				hayTeclas = False
				for event in pygame.event.get():
					if event.type == pygame.KEYDOWN or event.type == pygame.KEYUP:
						hayTeclas = True
				if hayTeclas:
					self.detectordeEventos()
					#pygame.event.clear()
			
				# Hay que cumplir con asyncore.loop de lo contrario se retrasa el clock pygame
				angulo, x, y = self.protagonista.get_posicion()
				self.buffer_de_salida += "%s %s %s %s %s%s" % (self.nombre, "Tanque", angulo, x, y, TERMINATOR)
				''' for bala in self.mis_balas:
					self.buffer_de_salida += "%s %s %s %s %s%s" % (self.nombre, "Bala", bala.angulo, bala.rect.x, bala.rect.y, TERMINATOR) '''

			self.actualizar()

		if self.estado == "Terminar":
			print "*** Cerrando pygame y Saliendo del Juego ***"
			pygame.quit()
			sys.exit()

	def detectordeEventos(self):

		teclas = pygame.key.get_pressed()

		# girar en movimiento
		if teclas[pygame.K_UP] and teclas[pygame.K_RIGHT]:# adelante - derecha
			self.protagonista.evento(pygame.K_UP)
			self.protagonista.evento(pygame.K_RIGHT)
		elif teclas[pygame.K_UP] and teclas[pygame.K_LEFT]:# adelante - izquierda
			self.protagonista.evento(pygame.K_UP)
			self.protagonista.evento(pygame.K_LEFT)
		elif teclas[pygame.K_DOWN] and teclas[pygame.K_RIGHT]:# atras - derecha
			self.protagonista.evento(pygame.K_DOWN)
			self.protagonista.evento(pygame.K_LEFT)
		elif teclas[pygame.K_DOWN] and teclas[pygame.K_LEFT]:# atras - izquierda
			self.protagonista.evento(pygame.K_DOWN)
			self.protagonista.evento(pygame.K_RIGHT)

		# moverse sin girar
		elif teclas[pygame.K_UP] and not teclas[pygame.K_DOWN]:# adelante
			self.protagonista.evento(pygame.K_UP)
		elif teclas[pygame.K_DOWN] and not teclas[pygame.K_UP]:# atras
			self.protagonista.evento(pygame.K_DOWN)

		# girar sin moverse
		elif teclas[pygame.K_RIGHT] and not teclas[pygame.K_LEFT]:# derecha
			self.protagonista.evento(pygame.K_RIGHT)
		elif teclas[pygame.K_LEFT] and not teclas[pygame.K_RIGHT]:# izquierda
			self.protagonista.evento(pygame.K_LEFT)

		
		if teclas[pygame.K_LCTRL]: # Disparar.
			pass
			'''
			if len(self.mis_balas) < 1:
				self.crear_disparo(self.nombre, self.protagonista.angulo, self.protagonista.rect.center)'''

		elif teclas[pygame.K_ESCAPE]: # salir
			#self.buffer_de_salida = "%s %s %s %s %s%s" % (self.nombre, "bye", 0, 0, 0, TERMINATOR)
			self.estado = "Terminar"
		else:
			pass


        def actualizar(self):
	# dibuja los sprites y el fondo
		self.tanques.clear(self.ventana, self.fondo)
		#self.todas_las_balas.clear(self.ventana, self.fondo)

		cambios = self.tanques.draw(self.ventana)
		
		'''for bala in self.todas_las_balas:
			x, y = bala.Actualizar()
			if x < 0 or x > self.resolucion_monitor[0] or y < 0 or y > self.resolucion_monitor[1]:
			# Si se sale de la pantalla se elimina
				bala.kill()

		cambios.extend( self.todas_las_balas.draw(self.ventana) ) '''
		
		pygame.display.update(cambios) # dibuja el personaje y las balas



# --------------------- Configuraciones ------------------------------
	def set_entorno(self):
	# Configurar entorno de video e inicializar los módulos necesarios
		pygame.init()
		pygame.display.set_mode(self.resolucion_monitor, 0, 0)
		pygame.event.set_blocked([MOUSEMOTION, MOUSEBUTTONUP, MOUSEBUTTONDOWN, JOYAXISMOTION, JOYBALLMOTION, JOYHATMOTION, JOYBUTTONUP, JOYBUTTONDOWN])
		pygame.event.set_allowed([QUIT, ACTIVEEVENT, KEYDOWN, KEYUP, VIDEORESIZE, VIDEOEXPOSE, USEREVENT])
		pygame.key.set_repeat(5, 5)
		self.clock = pygame.time.Clock() # un reloj pygame
		self.fondo = pygame.transform.scale(pygame.image.load("fondo.jpg"), self.resolucion_monitor).convert_alpha()
		self.ventana = pygame.display.get_surface() # la ventana del juego

		pygame.mouse.set_visible(False)

		pygame.mixer.init()
		self.sonido_disparo = pygame.mixer.Sound("disparo.wav")
